unit fMain;

interface

uses
  uInifiles, uStopWatch, uVclUtils,
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, Grids, ValEdit, ComCtrls, XPMan, MMSystem, Math,
  Buttons;

type
  TForm1 = class(TForm)
	time: TTimer;
	PageControl1: TPageControl;
	tabsheetTimer: TTabSheet;
	tabsheetPreferences: TTabSheet;
	valuelisteditorTimerType: TValueListEditor;
	labelNotifySound: TLabel;
	comboboxNotifySound: TComboBox;
	labelValues: TLabel;
	buttonSelectFile: TButton;
	buttonPlaySound: TButton;
	buttonReset: TButton;
	buttonStartStop: TButton;
	OpenDialog1: TOpenDialog;
	radiogroupTimerType: TRadioGroup;
	checkboxShowMilliSec: TCheckBox;
	image: TImage;
	checkboxAutoStart: TCheckBox;
	checkboxStayOnTop: TCheckBox;
	procedure FormCreate(Sender: TObject);
	procedure FormDestroy(Sender: TObject);
	procedure timeTimer(Sender: TObject);
	procedure buttonResetClick(Sender: TObject);
	procedure buttonSelectFileClick(Sender: TObject);
	procedure buttonPlaySoundClick(Sender: TObject);
	procedure PageControl1Change(Sender: TObject);
	procedure buttonStartStopClick(Sender: TObject);
	procedure checkboxStayOnTopClick(Sender: TObject);
	procedure speedbuttonPlaySoundClick(Sender: TObject);
  private
	{ Private 錾 }
	FNeedToNotify: Boolean;
	procedure IniSynchronize(Sender: TObject);
	procedure UpdateView(Sender: TObject);
  public
	{ Public 錾 }
  end;

var
  Form1: TForm1;

implementation
{$R *.dfm}

procedure TForm1.IniSynchronize(Sender: TObject);
begin
	with _ini do
	begin
		if Loading or (WindowState = wsNormal) then
		begin
			Left := Sync('Left', Left);
			Top := Sync('Top', Top);
			Width := Sync('Width', Width);
			Height := Sync('Height', Height);
		end;

		with valuelisteditorTimerType do
			SyncSection(Name, Strings);
		with comboboxNotifySound do
			SyncSection(Name, Items);

		Sync([
			checkboxShowMilliSec,
			checkboxAutoStart,
			checkboxStayOnTop,
			comboboxNotifySound,
			radiogroupTimerType,
			nil]);

	end;
end;

procedure TForm1.UpdateView(Sender: TObject);
begin
	if _stopwatch.Stopped then
		buttonStartStop.Caption := 'Jn(&S)'
	else
		buttonStartStop.Caption := '~(&S)';
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
	Caption := Application.Title;
	_ini.LoadFromFile(IniSynchronize);
	if checkboxAutoStart.Checked then
		buttonStartStop.Click
	else
    	UpdateView(Sender);
	PageControl1.ActivePage := tabsheetTimer;
	PageControl1.OnChange(PageControl1);
	FNeedToNotify := True;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
	_ini.SaveToFile(IniSynchronize);
end;

procedure TForm1.timeTimer(Sender: TObject);
var
	t, diff: Integer;
	s: String;
begin
	if radiogroupTimerType.ItemIndex < 0 then
		t := 0
	else
		t := 1000 * StrToIntDef(valuelisteditorTimerType.Strings.Values[valuelisteditorTimerType.Strings.Names[radiogroupTimerType.ItemIndex]], 0);

	diff := _stopwatch.MilliSec - t;
	if diff > 0  then
	begin
		if checkboxShowMilliSec.Checked then
			s := Format('o:%0.1fb', [diff / 1000])
		else
			s := Format('o:%db', [diff div 1000]);
		if (t > 0) and FNeedToNotify then
		begin
			buttonPlaySound.OnClick(buttonPlaySound);
			FNeedToNotify := False;
		end;
	end
	else
	begin
		if checkboxShowMilliSec.Checked then
			s := Format('c:%0.1fb', [-diff / 1000])
		else
			s := Format('c:%db', [-diff div 1000]);
	end;

	Application.Title := s;
	Caption := '[^C}[ - ' + s;
end;

procedure TForm1.PageControl1Change(Sender: TObject);
var
	i,idx: Integer;
	org: Boolean;
begin
	if PageControl1.ActivePage = tabsheetTimer then
	begin
		org := FNeedToNotify;
		try
			idx := radiogroupTimerType.ItemIndex;
			if idx < 0 then
				with radiogroupTimerType do
					idx := _ini.Read(Self.Name, Name, ItemIndex);
			radiogroupTimerType.Items.Clear;
			for i := 0 to valuelisteditorTimerType.Strings.Count - 1 do
				radiogroupTimerType.Items.add(valuelisteditorTimerType.Strings.Names[i]);
			radiogroupTimerType.ItemIndex := EnsureRange(idx, -1, radiogroupTimerType.Items.Count-1);
		finally
			FNeedToNotify := org;
		end;
	end;
end;

procedure TForm1.speedbuttonPlaySoundClick(Sender: TObject);
begin
	FNeedToNotify := True;
end;

procedure TForm1.checkboxStayOnTopClick(Sender: TObject);
begin
	if checkboxStayOnTop.Checked then
		FormStyle := fsStayOnTop
	else
    	FormStyle := fsNormal;
end;

procedure TForm1.buttonStartStopClick(Sender: TObject);
begin
	if _stopwatch.Stopped then
		_stopwatch.Start
	else
		_stopwatch.Stop;
	UpdateView(Sender);
end;

procedure TForm1.buttonPlaySoundClick(Sender: TObject);
begin
	Win32Check(PlaySound(PChar(comboboxNotifySound.Text), 0, SND_ASYNC));
    ComboBox_AddHistory(comboboxNotifySound, 32);
end;

procedure TForm1.buttonResetClick(Sender: TObject);
begin
	if _stopwatch.Stopped then
	begin
		_stopwatch.Reset;
		_stopwatch.Stop;
	end
	else
		_stopwatch.Reset;
	FNeedToNotify := True;
	UpdateView(Sender);
end;

procedure TForm1.buttonSelectFileClick(Sender: TObject);
begin
	OpenDialog1.FileName :=  comboboxNotifySound.Text;
	if OpenDialog1.Execute then
		comboboxNotifySound.Text := OpenDialog1.FileName;
end;

end.
