unit uInifiles;

interface
uses
	Windows, Classes, SysUtils, Inifiles,
	uSystem;
type
	TInifileEx = class;
	TSynchronizeComponent = procedure(ini: TInifileEx; Sender: TComponent);

	TSynchronizeMode = (smNone, smLoading, smSaving);
	TInifileEx = class
  private
	FFileName: String;
	FMode: TSynchronizeMode;
	FSection: String;
	// R|[lgwb_폜
	// 'btnStart' ==> 'Start' 擪̘A鏬폜
	// 'Button1' ==> 'Button1' 啶Ŏn܂Ăꍇ͕ς炸
	// 'button1' ==> 'button1' ׂď̏ꍇς炸
	function GetIdent(Ident: String): String;
	procedure SetFileName(const Value: String);
	procedure CheckArgs(AIdent: String);
  public
	ini: TInifile; // ƂƂڎgp
	property Section: String read FSection write FSection;

	// LoadFromFileOnSynchronizeCxgĂяoĂTrueƂȂ
	function Loading(AbortIfNotLoading: Boolean = False): Boolean;
	// SaveToFileOnSynxchronizeCxgĂяoĂTrueƂȂ
	function Saving(AbortIfNotSaving: Boolean = False): Boolean;

	constructor Create(AFileName: String); overload;
	destructor Destroy; override;
	property FileName: String read FFileName write SetFileName;
	procedure LoadFromFile(OnSynchronize: TNotifyEvent);
	procedure SaveToFile(OnSynchronize: TNotifyEvent);

	function Sync(Ident: String; Value: String): String; overload;
	function Read(Ident: String; Value: String): String; overload;
	procedure Write(Ident: String; Value: String); overload;

	function Sync(Ident: String; Value: Integer): Integer; overload;
	function Read(Ident: String; Value: Integer): Integer; overload;
	procedure Write(Ident: String; Value: Integer); overload;
	function Read(Section: String; Ident: String; Value: Integer): Integer; overload;
	procedure Write(Section: String; Ident: String; Value: Integer); overload;

	function Sync(Ident: String; Value: Boolean): Boolean; overload;
	function Read(Ident: String; Value: Boolean): Boolean; overload;
	procedure Write(Ident: String; Value: Boolean); overload;

	procedure Sync(Value: TComponent); overload;
	procedure Sync(Values: array of TComponent); overload;

	procedure SyncSection(Section: String; Value: TStrings);
	procedure ReadSection(Section: String; Value: TStrings);
	procedure WriteSection(Section: String; Value: TStrings);
(*
	function Sync(Ident: String; Value: @@): @@; overload;
	function Read(Ident: String; Value: @@): @@; overload;
	procedure Write(Ident: String; Value: @@); overload;
*)
  end;

procedure RegisterSynchronizer(AClass: TClass; Sync: TSynchronizeComponent);

var
	_ini: TInifileEx;

implementation

type
	TSyncItem = record
		AClass: TClass;
		Sync: TSynchronizeComponent;
	end;

var
	SyncItemList: array of TSyncItem;

procedure RegisterSynchronizer(AClass: TClass; Sync: TSynchronizeComponent);
begin
	SetLength(SyncItemList, Length(SyncItemList)+1);
	SyncItemList[Length(SyncItemList)-1].AClass := AClass;
	SyncItemList[Length(SyncItemList)-1].Sync := Sync;
end;

procedure TInifileEx.Sync(Value: TComponent);
var
	i: Integer;
begin
	if Value = nil then
		Exit;
//		EError.Error('TInifileEx.Sync', 'ValuenilB');

	for i := High(SyncItemList) downto Low(SyncItemList) do
		with SyncItemList[i] do
			if Value is AClass then
			begin
				Sync(Self, Value);
				Exit;
			end;

	EError.Error('TInifileEx.Sync', '%sɑ΂TSynchronizeComponento^ĂȂB',
		[Value.className]);
end;

procedure TInifileEx.Sync(Values: array of TComponent);
var
	i: Integer;
begin
	for i := Low(Values) to High(Values) do
		Sync(Values[i]);
end;

{ TInifileEx }

procedure TInifileEx.CheckArgs(AIdent: String);
begin
	if Section = '' then
		EError.Error('TInifileEx.CheckArgs', 'Section');
	if AIdent = '' then
		EError.Error('TInifileEx.CheckArgs', 'Ident');
end;

constructor TInifileEx.Create(AFileName: String);
begin
	FFileName := AFileName;
	ini := TIniFile.Create(FFileName);
    FSection := 'Settings';
end;

destructor TInifileEx.Destroy;
begin
	ini.Free;
  inherited;
end;

function TInifileEx.GetIdent(Ident: String): String;
var
	i: Integer;
begin
	for i := 1 to Length(Ident) do
		if Ident[i] in ['A'..'Z'] then
		begin
			Result := Copy(Ident, i, High(Integer));
			Exit;
		end;
	Result := Ident;
end;

function TInifileEx.Loading(AbortIfNotLoading: Boolean): Boolean;
begin
	Result := (FMode = smLoading);
	if not Result and AbortIfNotLoading then
		Abort;
end;

function TInifileEx.Saving(AbortIfNotSaving: Boolean): Boolean;
begin
	Result := (FMode = smSaving);
	if not Result and AbortIfNotSaving then
		Abort;
end;

procedure TInifileEx.LoadFromFile(OnSynchronize: TNotifyEvent);
var
	org: TSynchronizeMode;
begin
	if not Assigned(OnSynchronize) then
		EError.Error('TInifileEx.LoadFromFile', 'OnSynchronzew肳ĂȂB');

	org := FMode;
	try
		FMode := smLoading;
		OnSynchronize(Self);
	finally
		FMode := org;
	end;
end;

procedure TInifileEx.SaveToFile(OnSynchronize: TNotifyEvent);
var
	org: TSynchronizeMode;
begin
	if not Assigned(OnSynchronize) then
		EError.Error('TInifileEx.LoadFromFile', 'OnSynchronzew肳ĂȂB');

	org := FMode;
	try
		FMode := smSaving;
		OnSynchronize(Self);
	finally
		FMode := org;
	end;
end;

function TInifileEx.Read(Ident: String; Value: Integer): Integer;
begin
	CheckArgs(Ident);
	Result := ini.ReadInteger(Section, GetIdent(Ident), Value);
end;

function TInifileEx.Read(Section, Ident: String; Value: Integer): Integer;
begin
	CheckArgs(Ident);
	Result := ini.ReadInteger(Section, GetIdent(Ident), Value);
end;

function TInifileEx.Read(Ident, Value: String): String;
begin
	CheckArgs(Ident);
	Result := ini.ReadString(Section, GetIdent(Ident), Value);
end;

function TInifileEx.Read(Ident: String; Value: Boolean): Boolean;
begin
	CheckArgs(Ident);
	Result := ini.ReadBool(Section, GetIdent(Ident), Value);
end;

procedure TInifileEx.ReadSection(Section: String; Value: TStrings);
var
	buf: array[0..64*1024] of char;
	n: DWord;
	p: PChar;
begin
	Section := GetIdent(Section);
	CheckArgs('a');
	n := GetPrivateProfileSection(PChar(Section), @buf, Length(buf), PChar(FileName));
	if n = 0 then
    	Exit;
	p := buf;
	Value.Clear;
	while p^ <> #0 do
	begin
		Value.Add(p);
		Inc(p, StrLen(p) + 1);
	end;
end;

procedure TInifileEx.SetFileName(const Value: String);
begin
  FFileName := Value;
  ini.Free;
  ini := TInifile.Create(FFileName);
end;

function TInifileEx.Sync(Ident: String; Value: Boolean): Boolean;
begin
	Ident := GetIdent(Ident); CheckArgs(Ident);
	Result := Value;
	case FMode of
		smLoading: Result := ini.ReadBool(Section, Ident, Value);
		smSaving: ini.WriteBool(Section, Ident, Value);
		else EError.Fail;
	end;
end;

function TInifileEx.Sync(Ident: String; Value: Integer): Integer;
begin
	Ident := GetIdent(Ident); CheckArgs(Ident);
	Result := Value;
	case FMode of
		smLoading: Result := ini.ReadInteger(Section, Ident, Value);
		smSaving: ini.WriteInteger(Section, Ident, Value);
		else EError.Fail;
	end;
end;

function TInifileEx.Sync(Ident, Value: String): String;
begin
	Ident := GetIdent(Ident); CheckArgs(Ident);
	Result := Value;
	case FMode of
		smLoading: Result := ini.ReadString(Section, Ident, Value);
		smSaving: ini.WriteString(Section, Ident, Value);
		else EError.Fail;
	end;
end;

procedure TInifileEx.SyncSection(Section: String; Value: TStrings);
begin
	case FMode of
		smLoading: ReadSection(Section, Value);
		smSaving: WriteSection(Section, Value);
		else EError.Fail;
	end;
end;

procedure TInifileEx.Write(Section, Ident: String; Value: Integer);
begin
	CheckArgs(Ident);
	ini.WriteInteger(Section, Ident, Value);
end;

procedure TInifileEx.Write(Ident: String; Value: Integer);
begin
	CheckArgs(Ident);
	ini.WriteInteger(Section, Ident, Value);
end;

procedure TInifileEx.Write(Ident, Value: String);
begin
	CheckArgs(Ident);
	ini.WriteString(Section, GetIdent(Ident), Value);
end;

procedure TInifileEx.Write(Ident: String; Value: Boolean);
begin
	CheckArgs(Ident);
	ini.WriteBool(Section, GetIdent(Ident), Value);
end;

procedure TInifileEx.WriteSection(Section: String; Value: TStrings);
var
	buf: String;
	s: String;
begin
	Section := GetIdent(Section);
	CheckArgs('a');
    ini.EraseSection(Section);
	buf := '';
	for s in Value do
		buf := buf + s + #0;
	buf := buf + #0;
	WritePrivateProfileSection(PChar(Section), PChar(buf), PChar(FFileName));
end;

initialization
	_ini := TInifileEx.Create(ChangeFileExt(ParamStr(0), '.ini'));
finalization
	_ini.Free;
end.
